@@ -32,13 +32,9 @@
 #include <errno.h>
 #include <dirent.h>
 #include <unistd.h>  /* uid_t */
-#include <sys/time.h>  /* linux-specific, gives me strptime() */
-/* please note that because strptime() is not part of ANSI standard
-   C, you may need to define the macro _XOPEN_SOURCE = 1 (or use
-   -D_XOPEN_SOURCE when compiling). This macro tells gcc that
-   you want to use features of Linux beyond the strict ANSI
-   subset.
-*/
+#include <time.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>  /* contains all keyboard defs */
 #include "proctool.h"
@@ -229,17 +225,9 @@ gint handle_keypress_in_edit_win(GtkWidget *edit_window, GdkEventKey *event, gpo
 
 /* show the 'find' dialogue box */
 static void show_find_dialog_box(GtkWidget *calling_widget, gpointer func_data);
-
-/* actually do the finding */
-static void do_find(GtkWidget *calling_widget, gpointer func_data);
-
-/* handle a keypress in the find dialogue.
-   If <Return> was pressed, run do_find().
-   If <Esc> was pressed, destroy the find window. */
-gint handle_keypress_in_find_win(GtkWidget *find_window, GdkEventKey *event, gpointer func_data);
-
-/* clear the text widget in the find dialogue box */
-static void do_clear(GtkWidget *calling_widget, gpointer func_data);
+static void search (GtkWidget*, gpointer);
+static gint search_dlg_key_cb (GtkWidget*, GdkEventKey*, gpointer);
+static void clear_search (GtkWidget*, gpointer);
 
 /* make glbl_last_found_row equal -1 again */
 static void reset_last_found_row(GtkWidget *calling_widget, gpointer func_data);
@@ -373,9 +361,6 @@ static gint *glbl_int_ptr[COOKIE_COLUMNS];
 GdkPixmap *glbl_cookie_xpm;
 GdkBitmap *glbl_cookie_xpm_mask;
 
-/* the text entry field in the find dialog box */
-GtkWidget *glbl_find_text_entry = NULL;
-
 /* keeps track of the row of the last successful find */
 static gint glbl_last_found_row = -1;
 
@@ -407,7 +392,7 @@ int main (int argc, gchar *argv[]) {
 
     GdkColor *transparent = NULL;
 
-    GtkAcceleratorTable *accelerator_table;
+	GtkAccelGroup* accel_group;
 
     int i;
 
@@ -448,12 +433,10 @@ int main (int argc, gchar *argv[]) {
     gdk_window_set_icon(glbl_main_window->window, NULL,
 			glbl_cookie_xpm, glbl_cookie_xpm);
 
-    /* add accelerator_table to the window */
-    accelerator_table = gtk_accelerator_table_new();
-    gtk_window_add_accelerator_table(GTK_WINDOW(glbl_main_window),
-				     accelerator_table);
+	/* add accelerator_table to the window */
+	accel_group = gtk_accel_group_new();
+	gtk_window_add_accel_group(GTK_WINDOW(glbl_main_window), accel_group);
 
-    
     /* FILE MENU */
     
     /* Init the menu-widget, and remember -- never
@@ -479,10 +462,10 @@ int main (int argc, gchar *argv[]) {
     /* Do something interesting when the menuitem is selected */
     gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
 			      GTK_SIGNAL_FUNC(show_open_file_dialog_box), NULL);
-    /* install the accelerator */
-    gtk_widget_install_accelerator(menu_items, accelerator_table,
-				   "activate", 'O', GDK_CONTROL_MASK);
-	
+	/* install the accelerator */
+	gtk_widget_add_accelerator(menu_items, "activate", accel_group, 'O',
+							   GDK_CONTROL_MASK, 0);
+
     /* Show the widget */
     gtk_widget_show(menu_items);
 
@@ -500,9 +483,9 @@ int main (int argc, gchar *argv[]) {
     gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
 			      GTK_SIGNAL_FUNC(save_file), NULL);
     
-    /* install the accelerator */
-    gtk_widget_install_accelerator(menu_items, accelerator_table,
-				   "activate", 'S', GDK_CONTROL_MASK);
+	/* install the accelerator */
+	gtk_widget_add_accelerator(menu_items, "activate", accel_group, 'S',
+							   GDK_CONTROL_MASK, 0);
 	
     /* Show the widget */
     gtk_widget_show(menu_items);
@@ -553,11 +536,11 @@ int main (int argc, gchar *argv[]) {
     gtk_signal_connect(GTK_OBJECT (menu_items), "activate",
 		       GTK_SIGNAL_FUNC(gtk_main_quit),
 		       NULL);
-	
-    /* install the accelerator */
-    gtk_widget_install_accelerator(menu_items, accelerator_table,
-				   "activate", 'Q', GDK_CONTROL_MASK);
-	
+
+	/* install the accelerator */
+	gtk_widget_add_accelerator(menu_items, "activate", accel_group, 'Q',
+							   GDK_CONTROL_MASK, 0);
+
     /* Show the widget */
     gtk_widget_show(menu_items);
 
@@ -588,10 +571,10 @@ int main (int argc, gchar *argv[]) {
     gtk_signal_connect_object(GTK_OBJECT(glbl_cookie_menu_items), "activate",
 			      GTK_SIGNAL_FUNC(insert_row_glbl_clist),
 			      NULL);
-    /* install the accelerator */
-    gtk_widget_install_accelerator(glbl_cookie_menu_items, accelerator_table,
-				   "activate", 'N', GDK_CONTROL_MASK);
-	
+	/* install the accelerator */
+	gtk_widget_add_accelerator(glbl_cookie_menu_items, "activate", accel_group, 'N',
+							   GDK_CONTROL_MASK, 0);
+
     gtk_widget_show(glbl_cookie_menu_items);
     /*********/
 
@@ -604,10 +587,10 @@ int main (int argc, gchar *argv[]) {
     gtk_signal_connect_object(GTK_OBJECT(glbl_cookie_menu_items), "activate",
 			      GTK_SIGNAL_FUNC(show_edit_dialog_box),
 			      NULL);
-    /* install the accelerator */
-    gtk_widget_install_accelerator(glbl_cookie_menu_items, accelerator_table,
-				   "activate", 'E', GDK_CONTROL_MASK);
-	
+	/* install the accelerator */
+	gtk_widget_add_accelerator(glbl_cookie_menu_items, "activate", accel_group, 'E',
+							   GDK_CONTROL_MASK, 0);
+
     gtk_widget_show(glbl_cookie_menu_items);
     /*********/
 
@@ -705,10 +688,10 @@ int main (int argc, gchar *argv[]) {
 			      GTK_SIGNAL_FUNC(show_find_dialog_box),
 			      NULL);
     
-    /* install the accelerator */
-    gtk_widget_install_accelerator(edit_menu_items, accelerator_table,
-				   "activate", 'F', GDK_CONTROL_MASK);
-	
+	/* install the accelerator */
+	gtk_widget_add_accelerator(edit_menu_items, "activate", accel_group, 'F',
+							   GDK_CONTROL_MASK, 0);
+
     /* Show the widget */
     gtk_widget_show(edit_menu_items);
 
@@ -834,66 +817,36 @@ static void write_file (void) {
 static void show_about_dialog_box(GtkWidget *calling_widget, gpointer func_data) {
     static GtkWidget *about_window = NULL;
     GtkWidget *box1;
-    GtkWidget *box2;
-    GtkWidget *button;
-    GtkWidget *separator;
-
     GtkWidget *about_text_label;
 
-    if (!about_window) {
-	about_window = gtk_window_new (GTK_WINDOW_DIALOG);
-	gtk_window_position(GTK_WINDOW(about_window), GTK_WIN_POS_MOUSE);
+	if (!about_window) {
+		GtkWidget *btn;
 
+		about_window = gtk_dialog_new();
+		gtk_dialog_set_has_separator (GTK_DIALOG (about_window), FALSE);
+	gtk_window_position(GTK_WINDOW(about_window), GTK_WIN_POS_MOUSE);
 
 	gtk_signal_connect (GTK_OBJECT(about_window), "destroy",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroyed),
 			    &about_window);
 
-	/* REALLY IMPORTANT NOTE! gtk_widget_destroyed (notice the
-	   past tense) seems to do something a whole lot different
-	   than gtk_widget_destroy (notice the present tense)
-	*/
-
-
 	gtk_window_set_title (GTK_WINDOW (about_window), "About gtkcookie");
 	gtk_container_border_width (GTK_CONTAINER (about_window), 0);
 
-
-	box1 = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (about_window), box1);
+		box1 = gtk_vbox_new (FALSE, 10);
+		gtk_container_set_border_width (GTK_CONTAINER (box1), 10);
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (about_window)->vbox), box1);
 	gtk_widget_show (box1);
 
-
-	box2 = gtk_vbox_new (FALSE, 10);
-	gtk_container_border_width (GTK_CONTAINER (box2), 10);
-	gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
-	gtk_widget_show (box2);
-
 	about_text_label = gtk_label_new("gtkcookie, version 0.03\n\nCode copyright (c) 1998 Manni Wood.\nCookie icons copyright (c) 1998 Glenn Matheson.\n\ngtkcookie comes with ABSOLUTELY NO WARRANTY.\nThis is free software, and you are welcome\nto redistribute it under certain conditions;\nfor details, see the file named 'COPYING' that\ncame with the source code of this programme.\n\nNetscape is a registered trademark of\nNetscape Communications Corporation\nin the United States and other countries.");
-	gtk_box_pack_start (GTK_BOX (box2), about_text_label, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (box1), about_text_label, TRUE, TRUE, 0);
 	gtk_widget_show (about_text_label);
 
-	separator = gtk_hseparator_new ();
-	gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
-	gtk_widget_show (separator);
-
-
-	box2 = gtk_vbox_new (FALSE, 10);
-	gtk_container_border_width (GTK_CONTAINER (box2), 10);
-	gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
-	gtk_widget_show (box2);
-
-
-	
-	button = gtk_button_new_with_label ("Close");
-	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-				   GTK_SIGNAL_FUNC(gtk_widget_destroy),
-				   GTK_OBJECT (about_window));
-	gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-	gtk_widget_grab_default (button);
-	gtk_widget_show (button);
-    }
+		btn = gtk_dialog_add_button (GTK_DIALOG (about_window), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+		g_signal_connect_swapped (G_OBJECT (btn), "clicked", G_CALLBACK (gtk_widget_destroy), G_OBJECT (about_window));
+		gtk_widget_grab_default (btn);
+		gtk_widget_show (btn);
+	}
 
     if (!GTK_WIDGET_VISIBLE (about_window)) {
 	gtk_widget_show (about_window);
@@ -902,10 +855,10 @@ static void show_about_dialog_box(GtkWidget *calling_widget, gpointer func_data)
     }
 }
 
-static void show_find_dialog_box(GtkWidget *calling_widget, gpointer func_data) {
-    static GtkWidget *find_window = NULL;
+static void
+show_find_dialog_box (GtkWidget* wdg, gpointer data) {
+	static GtkWidget *search_dlg = NULL;
     GtkWidget *box1;
-    GtkWidget *hbox3;
     GtkWidget *hbox2;
     GtkWidget *hbox1;
 
@@ -913,47 +866,29 @@ static void show_find_dialog_box(GtkWidget *calling_widget, gpointer func_data)
 
     GtkWidget *case_sensitive_tf_checkbox;
     
-    GtkWidget *separator;
-
-    GtkWidget *find_button;
-    GtkWidget *clear_button;
-    GtkWidget *close_button;
-
     /* each time the find dialog box is opened, set glbl_last_found_row
        to zero, to indicate that whatever the user is about to search on,
        it is the first time he will be searching for that string */
     glbl_last_found_row = -1;
 
-    if (!find_window) {
-	find_window = gtk_window_new (GTK_WINDOW_DIALOG);
-	gtk_window_position(GTK_WINDOW(find_window), GTK_WIN_POS_MOUSE);
+	if (!search_dlg) {
+		GtkWidget *btn, *needle;
 
+		search_dlg = gtk_dialog_new();
+		gtk_window_position (GTK_WINDOW (search_dlg), GTK_WIN_POS_MOUSE);
 
-	gtk_signal_connect (GTK_OBJECT(find_window), "destroy",
-			    GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-			    &find_window);
-
+		g_signal_connect (G_OBJECT (search_dlg), "destroy", G_CALLBACK (gtk_widget_destroyed), &search_dlg);
 	/* REALLY IMPORTANT NOTE! gtk_widget_destroyed (notice the
 	   past tense) seems to do something a whole lot different
 	   than gtk_widget_destroy (notice the present tense)
 	*/
 
-
-	gtk_window_set_title (GTK_WINDOW (find_window), "gtkcookie Search");
-	gtk_container_border_width (GTK_CONTAINER (find_window), 0);
-
-	/* hitting any keyboard key over the find window will call
-	   a function that checks to see if the key was <Return> and,
-	   if so, call do_find() */
-	gtk_signal_connect(GTK_OBJECT(find_window), "key_press_event",
-			   GTK_SIGNAL_FUNC(handle_keypress_in_find_win),
-			   NULL);
+		gtk_window_set_title (GTK_WINDOW (search_dlg), _("Find cookie"));
 
 	box1 = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (find_window), box1);
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (search_dlg)->vbox), box1);
 	gtk_widget_show (box1);
 
-
 	hbox1 = gtk_hbox_new (FALSE, 10);
 	gtk_container_border_width (GTK_CONTAINER (hbox1), 10);
 	gtk_box_pack_start (GTK_BOX (box1), hbox1, TRUE, TRUE, 0);
@@ -964,23 +899,11 @@ static void show_find_dialog_box(GtkWidget *calling_widget, gpointer func_data)
 	gtk_box_pack_start (GTK_BOX (hbox1), find_text_label, TRUE, TRUE, 0);
 	gtk_widget_show (find_text_label);
 
-	glbl_find_text_entry = gtk_entry_new ();
-	gtk_signal_connect(GTK_OBJECT(glbl_find_text_entry), "changed",
-			   GTK_SIGNAL_FUNC(reset_last_found_row),
-			   NULL);
-	/* I'm a bit worried about what happens to the text inside
-	   glbl_find_text_entry when this window is deleted. Is it
-	   de-allocated, or memory leaked? */
- 	gtk_box_pack_start (GTK_BOX (hbox1), glbl_find_text_entry, TRUE, TRUE, 0);
-	/* ensure the text entry field grabs focus, because I don't want
-	   the user to have to click on the entry with his mouse
-	   to bring it into focus so that he can start typing inside it;
-	   it should automatically be in focus as soon as the window
-	   is shown */
-	gtk_widget_grab_focus(glbl_find_text_entry);
-	gtk_widget_show (glbl_find_text_entry);
-
-	/********************/
+		needle = gtk_entry_new();
+		g_signal_connect (G_OBJECT (needle), "changed", G_CALLBACK (reset_last_found_row), NULL);
+		gtk_box_pack_start (GTK_BOX (hbox1), needle, TRUE, TRUE, 0);
+		gtk_widget_grab_focus (needle);
+		gtk_widget_show (needle);
 
 	hbox2 = gtk_hbox_new (FALSE, 10);
 	gtk_container_border_width (GTK_CONTAINER (hbox2), 10);
@@ -1009,64 +932,40 @@ static void show_find_dialog_box(GtkWidget *calling_widget, gpointer func_data)
 	gtk_box_pack_start (GTK_BOX (hbox2), case_sensitive_tf_checkbox, FALSE, TRUE, 0);
 	gtk_widget_show (case_sensitive_tf_checkbox);
 	/********************/
-	
-	separator = gtk_hseparator_new ();
-	gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
-	gtk_widget_show (separator);
 
+		btn = gtk_dialog_add_button (GTK_DIALOG (search_dlg), GTK_STOCK_FIND, GTK_RESPONSE_ACCEPT);
+		g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (search), needle);
+		gtk_widget_show (btn);
 
-	hbox3 = gtk_hbox_new (FALSE, 10);
-	gtk_container_border_width (GTK_CONTAINER (hbox3), 10);
-	gtk_box_pack_start (GTK_BOX (box1), hbox3, FALSE, TRUE, 0);
-	gtk_widget_show (hbox3);
+		btn = gtk_dialog_add_button (GTK_DIALOG (search_dlg), GTK_STOCK_CLEAR, GTK_RESPONSE_REJECT);
+		g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (clear_search), needle);
+		gtk_widget_show (btn);
 
+		btn = gtk_dialog_add_button (GTK_DIALOG (search_dlg), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+		g_signal_connect_swapped (G_OBJECT (btn), "clicked", G_CALLBACK (gtk_widget_destroy), G_OBJECT (search_dlg));
+		gtk_widget_show (btn);
 
-	find_button = gtk_button_new_with_label ("Find");
-	gtk_signal_connect_object (GTK_OBJECT (find_button), "clicked",
-				   GTK_SIGNAL_FUNC(do_find),
-				   NULL );
-	gtk_box_pack_start (GTK_BOX (hbox3), find_button, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS (find_button, GTK_CAN_DEFAULT);
-	gtk_widget_grab_default (find_button);
-	gtk_widget_show (find_button);
+		/* check if the user hit Return and if so, call search() */
+		g_signal_connect (G_OBJECT (search_dlg), "key_press_event", G_CALLBACK (search_dlg_key_cb), needle);
+	}
 
-
-	clear_button = gtk_button_new_with_label ("Clear");
-	gtk_signal_connect_object (GTK_OBJECT (clear_button), "clicked",
-				   GTK_SIGNAL_FUNC(do_clear),
-				   NULL );
-	gtk_box_pack_start (GTK_BOX (hbox3), clear_button, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS (clear_button, GTK_CAN_DEFAULT);
-	gtk_widget_show (clear_button);
-
-
-	close_button = gtk_button_new_with_label ("Close");
-	gtk_signal_connect_object (GTK_OBJECT (close_button), "clicked",
-				   GTK_SIGNAL_FUNC(gtk_widget_destroy),
-				   GTK_OBJECT (find_window));
-	gtk_box_pack_start (GTK_BOX (hbox3), close_button, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
-	gtk_widget_show (close_button);
-    }
-
-    if (!GTK_WIDGET_VISIBLE (find_window)) {
-	gtk_widget_show (find_window);
+	if (!GTK_WIDGET_VISIBLE (search_dlg)) {
+		gtk_widget_show (search_dlg);
     } else {
-	gtk_widget_destroy (find_window);
+		gtk_widget_destroy (search_dlg);
     }
 }
 
 static void show_error_dialog_box (gchar *message) {
     static GtkWidget *error_window = NULL;
     GtkWidget *box1;
-    GtkWidget *box2;
-    GtkWidget *button;
-    GtkWidget *separator;
-
     GtkWidget *about_text_label;
 
-    if (!error_window) {
-	error_window = gtk_window_new (GTK_WINDOW_DIALOG);
+	if (!error_window) {
+		GtkWidget *btn;
+
+		error_window = gtk_dialog_new();
+		gtk_dialog_set_has_separator (GTK_DIALOG (error_window), FALSE);
 	gtk_window_position(GTK_WINDOW(error_window), GTK_WIN_POS_MOUSE);
 
 	gtk_signal_connect (GTK_OBJECT (error_window), "destroy",
@@ -1076,42 +975,20 @@ static void show_error_dialog_box (gchar *message) {
 	gtk_window_set_title (GTK_WINDOW (error_window), "Error");
 	gtk_container_border_width (GTK_CONTAINER (error_window), 0);
 
-	box1 = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (error_window), box1);
+		box1 = gtk_vbox_new (FALSE, 10);
+		gtk_container_border_width (GTK_CONTAINER (box1), 10);
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (error_window)->vbox), box1);
 	gtk_widget_show (box1);
 
-
-	box2 = gtk_vbox_new (FALSE, 10);
-	gtk_container_border_width (GTK_CONTAINER (box2), 10);
-	gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
-	gtk_widget_show (box2);
-
 	about_text_label = gtk_label_new(message);
-	gtk_box_pack_start (GTK_BOX (box2), about_text_label, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (box1), about_text_label, TRUE, TRUE, 0);
 	gtk_widget_show (about_text_label);
 
-	separator = gtk_hseparator_new ();
-	gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
-	gtk_widget_show (separator);
-
-
-	box2 = gtk_vbox_new (FALSE, 10);
-	gtk_container_border_width (GTK_CONTAINER (box2), 10);
-	gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
-	gtk_widget_show (box2);
-
-
-	button = gtk_button_new_with_label ("OK");
-
-	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-	 			   GTK_SIGNAL_FUNC(gtk_widget_destroy),
-				   GTK_OBJECT (error_window));
-
-	gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-	gtk_widget_grab_default (button);
-	gtk_widget_show (button);
-    }
+		btn = gtk_dialog_add_button (GTK_DIALOG (error_window), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+		g_signal_connect_swapped (G_OBJECT (btn), "clicked", G_CALLBACK (gtk_widget_destroy), G_OBJECT (error_window));
+		gtk_widget_grab_default (btn);
+		gtk_widget_show (btn);
+	}
 
     if (!GTK_WIDGET_VISIBLE (error_window)) {
 	/* gtk_widget_realize (error_window); */
@@ -1301,6 +1178,8 @@ static void insert_row_glbl_clist (GtkWidget *calling_widget, gpointer func_data
 
 
 static void create_glbl_clist (GtkWidget *box1) {
+	/* ScrolledWindow to be put around glbl_clist */
+	GtkWidget* sw_clist;
 
     /* create GtkCList here so we have a pointer to throw at the 
      * button callbacks -- more is done with it later */
@@ -1349,14 +1228,15 @@ static void create_glbl_clist (GtkWidget *box1) {
 
     /* what is a selection mode? */
     gtk_clist_set_selection_mode (GTK_CLIST (glbl_clist), GTK_SELECTION_BROWSE);
-    /* what is a policy? */
-    gtk_clist_set_policy (GTK_CLIST (glbl_clist), 
-			  GTK_POLICY_AUTOMATIC,
-			  GTK_POLICY_AUTOMATIC);
+	/* what is a policy? */
+	sw_clist = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw_clist), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(sw_clist), glbl_clist);
 
-    gtk_container_border_width (GTK_CONTAINER (glbl_clist), 0);
-    gtk_box_pack_start (GTK_BOX (box1), glbl_clist, TRUE, TRUE, 0);
-    gtk_widget_show (glbl_clist);
+	gtk_container_border_width (GTK_CONTAINER (glbl_clist), 0);
+	gtk_box_pack_start (GTK_BOX (box1), sw_clist, TRUE, TRUE, 0);
+	gtk_widget_show (glbl_clist);
+	gtk_widget_show(sw_clist);
 }
 
 /* POPULATE GLBL_CLIST */
@@ -2225,28 +2105,22 @@ static void check_for_running_netscape(void) {
     }
 }
 
-static void do_find(GtkWidget *calling_widget, gpointer func_data) {
+static void
+search (GtkWidget* wdg, gpointer data) {
     gint row;
     gint column;
-    gchar *find_text_entry_contents, *cell_contents;
+	gchar *needle, *cell_contents;
 
     /* time to try to use gstrings, methinks... */
     /* oooh, gstrings are sexy! */
     GString *cell_str;  /* the string from a cell in the clist */
     GString *find_str;  /* the string in the "find" text entry box */
 
-    /* get a pointer to whatever the user entered in the text entry */
-    find_text_entry_contents = gtk_entry_get_text(GTK_ENTRY(glbl_find_text_entry));
-    /* if the user entered no text, do nothing */
-    if (strlen(find_text_entry_contents) == 0) {
-	return;
-    }
-    
-    /* we need to copy construct find_text_entry_contents, so that when we
-       uppercase find_text_entry_contents, we are not uppercasing the actual
-       clist cell data that find_text_entry_contents points to! */
-    find_str = g_string_new("");
-    g_string_append(find_str, find_text_entry_contents);
+	needle = (gchar*) gtk_entry_get_text (GTK_ENTRY (data));
+	if (!strlen (needle)) {
+		return;
+	}
+	find_str = g_string_new (needle);
 
     /* if the search is case-insensitive, make everything in
        find_str uppercase */
@@ -2310,8 +2184,9 @@ static void do_find(GtkWidget *calling_widget, gpointer func_data) {
     show_error_dialog_box("Not found.");
 }
 
-static void do_clear(GtkWidget *calling_widget, gpointer func_data) {
-    gtk_entry_set_text(GTK_ENTRY(glbl_find_text_entry), "");
+static void
+clear_search (GtkWidget* wdg, gpointer data) {
+	gtk_entry_set_text (GTK_ENTRY (data), "");
     /* the last found row is rendered invalid when the user clears
        the text box, so reset that variable to -1 */
     glbl_last_found_row = -1;
@@ -2461,17 +2336,13 @@ static void set_tf_case_sensitive_search(GtkWidget *toggle_button, gpointer func
     }
 }
 
-gint handle_keypress_in_find_win(GtkWidget *find_window, GdkEventKey *event, gpointer func_data) {
-    if (event->keyval == GDK_Return) {
-	/* do_find is also called as a callback function, where it
-	   gets pointers to the calling widget (arg1) and any extra
-	   function data (arg2), neither of which is necessary here,
-	   so go with 2 NULLs */
-	do_find(NULL, NULL);
-	return TRUE;  /* tell calling code that we have handled this event */
-    }
-    if (event->keyval == GDK_Escape) {
-	gtk_widget_destroy(find_window);
+static gint
+search_dlg_key_cb (GtkWidget* wdg, GdkEventKey* ev, gpointer data) {
+	if (ev->keyval == GDK_Return) {
+		search (wdg, data);
+		return TRUE;
+	} else if (ev->keyval == GDK_Escape) {
+		gtk_widget_destroy (wdg);
 	return TRUE;  /* tell calling code that we have handled this event */
     }
     return FALSE;
